@@ -28,194 +28,7 @@ import vstruct.defs.elf as vs_elf
 
 logger = logging.getLogger(__name__)
 
-
-class ElfReloc:
-    """
-    Elf relocation entries consist mostly of "fixup" address which
-    are taken care of by the loader at runtime.  Things like
-    GOT entries, PLT jmp codes etc all have an Elf relocation
-    entry.
-    """
-
-    def __init__(self):
-        self.name = ""
-
-    def __repr__(self):
-        return "reloc: @%s %d %s" % (hex(self.r_offset), self.getType(), self.getName())
-
-    def setName(self, name):
-        self.name = name
-
-    def getName(self):
-        return self.name
-
-    def getType(self):
-        return self.r_info & 0xff
-
-
-class Elf32Reloc(ElfReloc, vs_elf.Elf32Reloc):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Reloc.__init__(self, bigend=bigend)
-        ElfReloc.__init__(self)
-
-    def getSymTabIndex(self):
-        return self.r_info >> 8
-
-
-class Elf32Reloca(ElfReloc, vs_elf.Elf32Reloca):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Reloca.__init__(self, bigend=bigend)
-        ElfReloc.__init__(self)
-
-    def getSymTabIndex(self):
-        return self.r_info >> 8
-
-
-class Elf64Reloc(ElfReloc, vs_elf.Elf64Reloc):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Reloc.__init__(self, bigend=bigend)
-        ElfReloc.__init__(self)
-
-    def getSymTabIndex(self):
-        return self.r_info >> 32
-
-
-class Elf64Reloca(ElfReloc, vs_elf.Elf64Reloca):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Reloca.__init__(self, bigend=bigend)
-        ElfReloc.__init__(self)
-
-    def getSymTabIndex(self):
-        return self.r_info >> 32
-
-
-class ElfDynamic:
-    """
-    An object to represent an Elf dynamic entry.
-    (linker/loader directives)
-    """
-    has_string = [DT_NEEDED, DT_SONAME]
-
-    def __init__(self, bytes=None):
-        self.name = ""
-
-    def __repr__(self):
-        name = self.getName()
-        if not name:
-            name = hex(self.d_value)
-        return "%s %s" % (name, self.getTypeName())
-
-    def getName(self):
-        return self.name
-
-    def setName(self, name):
-        self.name = name
-
-    def getTypeName(self):
-        return dt_types.get(self.d_tag, "Unknown: %s"%hex(self.d_tag))
-
-class Elf32Dynamic(ElfDynamic, vs_elf.Elf32Dynamic):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Dynamic.__init__(self, bigend=bigend)
-        ElfDynamic.__init__(self)
-
-class Elf64Dynamic(ElfDynamic, vs_elf.Elf64Dynamic):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Dynamic.__init__(self, bigend=bigend)
-        ElfDynamic.__init__(self)
-
-class ElfSymbol:
-    def __init__(self):
-        self.name = ""
-
-    def getInfoType(self):
-        return self.st_info & 0xf
-
-    def getInfoBind(self):
-        return self.st_info >> 4
-
-    def __cmp__(self, other):
-        if self.st_value > other.st_value:
-            return 1
-        return -1
-
-    def setName(self, name):
-        self.name = name
-
-    def getName(self):
-        return self.name
-
-    def __repr__(self):
-        return "0x%.8x %d %s" % (self.st_value, self.st_size, self.name)
-
-class Elf32Symbol(ElfSymbol, vs_elf.Elf32Symbol):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Symbol.__init__(self, bigend=bigend)
-        ElfSymbol.__init__(self)
-
-class Elf64Symbol(ElfSymbol, vs_elf.Elf64Symbol):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Symbol.__init__(self, bigend=bigend)
-        ElfSymbol.__init__(self)
-
-class ElfPheader:
-    def __init__(self):
-        pass
-
-    def getTypeName(self):
-        return ph_types.get(self.p_type, "Unknown")
-
-    def __repr__(self):
-        return '[%35s] VMA: 0x%.8x  offset: %8d  memsize: %8d  align: %8d  (filesz: %8d)  flags: %x' % (
-            self.getTypeName(),
-            self.p_vaddr,
-            self.p_offset,
-            self.p_memsz,
-            self.p_align,
-            self.p_filesz,
-            self.p_flags)
-
-class Elf32Pheader(ElfPheader, vs_elf.Elf32Pheader):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Pheader.__init__(self, bigend=bigend)
-        ElfPheader.__init__(self)
-
-class Elf64Pheader(ElfPheader, vs_elf.Elf64Pheader):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Pheader.__init__(self, bigend=bigend)
-        ElfPheader.__init__(self)
-
-class ElfSection:
-    def __init__(self):
-        self.name = ''
-
-    def setName(self, name):
-        self.name = name
-
-    def getName(self):
-        return self.name
-
-    def __repr__(self):
-        flags = [name for idx, name in sh_flags.items() if idx & self.sh_flags]
-
-        return 'Elf Sec: [%20s] @0x%.8x (%8d) [ent/size: %8d/%8d] [align: %8d] [%s]' % (
-                self.name,
-                self.sh_addr,
-                self.sh_offset,
-                self.sh_entsize,
-                self.sh_size,
-                self.sh_addralign,
-                'Flags: ' + ', '.join(flags))
-
-class Elf32Section(ElfSection, vs_elf.Elf32Section):
-    def __init__(self, bigend=False):
-        vs_elf.Elf32Section.__init__(self, bigend=bigend)
-        ElfSection.__init__(self)
-
-class Elf64Section(ElfSection, vs_elf.Elf64Section):
-    def __init__(self, bigend=False):
-        vs_elf.Elf64Section.__init__(self, bigend=bigend)
-        ElfSection.__init__(self)
+HAS_STRING = [DT_NEEDED, DT_SONAME]
 
 class Elf(vs_elf.Elf32, vs_elf.Elf64):
     def __init__(self, fd, inmem=False):
@@ -236,26 +49,27 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         # if e_data == 1, then 32 bit, if e_data == 2, 64bit
         bigend = (e.e_data == ELFDATA2MSB)
 
-        #Parse 32bit header
+        # Parse 32bit header
         if e.e_class == ELFCLASS32:
             vs_elf.Elf32.__init__(self, bigend=bigend)
             self.bits = 32
             self.psize = 4
 
-            self._cls_reloc  = Elf32Reloc
-            self._cls_reloca = Elf32Reloca
-            self._cls_symbol = Elf32Symbol
-            self._cls_section = Elf32Section
-        #Parse 64bit header
+            self._cls_reloc  = vs_elf.Elf32Reloc
+            self._cls_reloca = vs_elf.Elf32Reloca
+            self._cls_symbol = vs_elf.Elf32Symbol
+            self._cls_section = vs_elf.Elf32Section
+
+        # Parse 64bit header
         elif e.e_class == ELFCLASS64:
             vs_elf.Elf64.__init__(self, bigend=bigend)
             self.bits = 64
             self.psize = 8
 
-            self._cls_reloc  = Elf64Reloc
-            self._cls_reloca = Elf64Reloca
-            self._cls_symbol = Elf64Symbol
-            self._cls_section = Elf64Section
+            self._cls_reloc  = vs_elf.Elf64Reloc
+            self._cls_reloca = vs_elf.Elf64Reloca
+            self._cls_symbol = vs_elf.Elf64Symbol
+            self._cls_section = vs_elf.Elf64Section
         else:
             raise Exception('Unrecognized e_class: %d' % e.e_class)
 
@@ -273,12 +87,13 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         else:
             self.r_types = {}
 
+        self.dyns = {}
         self.pheaders = []
         self.sections = []
         self.secnames = {}
         self.symbols  = []
         self.relocs   = []
-        self.relocvas = []
+        self.relocvas = set([])
         self.symbols_by_name = {}
         self.symbols_by_addr = {}
         self.dynamics = []      # deprecated - 2019-10-21
@@ -346,9 +161,9 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
             plen = self.e_phentsize
             for i in range(self.e_phnum):
                 if self.bits == 32:
-                    pgm = Elf32Pheader(bigend=self.bigend)
+                    pgm = vs_elf.Elf32Pheader(bigend=self.bigend)
                 elif self.bits == 64:
-                    pgm = Elf64Pheader(bigend=self.bigend)
+                    pgm = vs_elf.Elf64Pheader(bigend=self.bigend)
                 else:
                     raise Exception('Platform not supported: %d' % (self.bits))
 
@@ -450,14 +265,14 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         dynbytes = self.getSectionBytes('.dynamic')
         while dynbytes:
             if self.bits == 32:
-                dyn = Elf32Dynamic(bigend=self.bigend)
+                dyn = vs_elf.Elf32Dynamic(bigend=self.bigend)
             elif self.bits == 64:
-                dyn = Elf64Dynamic(bigend=self.bigend)
+                dyn = vs_elf.Elf64Dynamic(bigend=self.bigend)
             else:
                 raise Exception('Platform not supported: %d' % (self.bits))
             dyn.vsParse(dynbytes)
 
-            if dyn.d_tag in Elf32Dynamic.has_string:
+            if dyn.d_tag in HAS_STRING:
                 name = self.getStrtabString(dyn.d_value, ".dynstr")
                 dyn.setName(name)
 
@@ -476,16 +291,15 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         This must be run before most Dynamic-data accessors like getDynStrtabString(),
         getDynSymTabInfo(), etc..
         '''
-        self.dyns = {}
         dynbytes = self.getDynBytes()
         if dynbytes is None:
             return
 
         while dynbytes:
             if self.bits == 32:
-                dyn = Elf32Dynamic(bigend=self.bigend)
+                dyn = vs_elf.Elf32Dynamic(bigend=self.bigend)
             elif self.bits == 64:
-                dyn = Elf64Dynamic(bigend=self.bigend)
+                dyn = vs_elf.Elf64Dynamic(bigend=self.bigend)
             else:
                 raise Exception('Platform not supported: %d' % (self.bits))
 
@@ -508,19 +322,19 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
 
     def _parseDynStrs(self):
         # setup STRTAB for string recovery:
-        dynstrtab = self.dyns.get(DT_STRTAB)
+        dynstrtabva = self.dyns.get(DT_STRTAB)
         strsz = self.dyns.get(DT_STRSZ)
-        if dynstrtab is None or strsz is None:
-            logger.info('no dynamic string tableinfo found: DT_STRTAB: %r  DT_STRSZ: %r', dynstrtab, strsz)
+        if dynstrtabva is None or strsz is None:
+            logger.info('no dynamic string tableinfo found: DT_STRTAB: %r  DT_STRSZ: %r', dynstrtabva, strsz)
             return
 
         if self.dynstrtabmeta != (None, None):
             curtab = self.dynstrtabmeta[0]
             logger.warning('wtf?  multiple dynamic string tables?  old: 0x%x  new: 0x%x', curtab, rva)
 
-        strtabbytes = self.readAtRva(dynstrtab, strsz)
+        strtabbytes = self.readAtRva(dynstrtabva, strsz)
 
-        self.dynstrtabmeta = (dynstrtab, strsz)
+        self.dynstrtabmeta = (dynstrtabva, strsz)
         self.dynstrtab = strtabbytes.split(b'\0')
 
         # since our string table should certainly end in '\0', we'll have an empty string
@@ -529,16 +343,9 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         if len(self.dynstrtab) and not len(self.dynstrtab[-1]):
             self.dynstrtab.pop()
 
-        self.dynsymtabct = len(self.dynstrtab)  # cheat: there is a 1-to-1 relationship between symbols and strings in these tables
-        # if "DT_SONAME" is within this string table, there are no symbols to match that or thereafter:
-        soname = self.dyns.get(DT_SONAME)
-        if soname is not None and soname != -1 and soname < strsz:
-            dynsymstrs = strtabbytes[:soname].split(b'\0')
-            self.dynsymtabct = len(dynsymstrs) - 1
-
         # setup names for the dynamics table entries
         for dyn in self.dynamics:
-            if dyn.d_tag in Elf32Dynamic.has_string:
+            if dyn.d_tag in HAS_STRING:
                 name = self.getDynStrtabString(dyn.d_value)
                 dyn.setName(name)
 
@@ -575,10 +382,10 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
             syment = self.readAtRva(symtabrva + dsoff, symsz)
             sym = self._cls_symbol(bigend=self.bigend)
             sym.vsParse(syment)
-            if sym.st_info & 0xf not in st_info_type:
+            if sym.getInfoType() not in st_info_type:
                 break
 
-            if sym.st_info >> 4 not in st_info_bind:
+            if sym.getInfoBind() not in st_info_bind:
                 break
 
             name = self.getDynStrtabString(sym.st_name)
@@ -653,8 +460,8 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
             sym = self.getDynSymbol(index)
             if sym is not None:
                 reloc.setName( sym.getName() )
-            self.relocs.append(reloc)
-            self.relocvas.append(reloc.r_offset)
+            self.relocs.append((None, reloc))
+            self.relocvas.add((None, reloc.r_offset))
 
     def _parseSectionRelocs(self):
         """
@@ -669,7 +476,7 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         jmprel, pltrel, pltrelsz = self.getDynPltRelInfo()
         dynrels = (rel, rela, jmprel)
 
-        for sec in self.sections:
+        for secidx, sec in enumerate(self.sections):
             if sec.sh_type not in (SHT_REL, SHT_RELA):
                 continue
 
@@ -691,16 +498,20 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
                 index = reloc.getSymTabIndex()
                 if index < len(self.dynamic_symbols):
                     sym = self.dynamic_symbols[index]
-                    reloc.setName( sym.getName() )
+                    reloc.setName(sym.getName())
 
-                if reloc.r_offset in self.relocvas:
-                    # FIXME: This line is hit sever tens of thousands of times during parsing
+                # key on the symbol table index because relocatable elf files use r_offset
+                # as an actual offset and not a virtual address like executable images
+                # For all other binaries, let dynamics win
+                # Ref: https://docs.oracle.com/cd/E23824_01/html/819-0690/chapter6-54839.html 
+                key = (index, reloc.r_offset)
+                if key in self.relocvas or (None, reloc.r_offset) in self.relocvas:
                     logger.debug('duplicate relocation (section): %s', reloc)
                     continue
 
                 logger.info('section reloc: %s', reloc)
-                self.relocs.append(reloc)
-                self.relocvas.append(reloc.r_offset)
+                self.relocs.append((secidx, reloc))
+                self.relocvas.add(key)
 
     def getBaseAddress(self):
         """
@@ -712,7 +523,7 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
 
         # If it's a shared object and *not* prelinked,
         # we need to select a base address for it
-        # FIXME find non-coliding addr in workspace
+        # FIXME find non-colliding addr in workspace
         if shrd and not plnk:
             return 0x02000000
 
@@ -732,7 +543,10 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
                 base = pgm.p_vaddr
 
         if base is None:
-            base = 0x20000000
+            if self.isRelocatable():
+                base = 0
+            else:
+                base = 0x20000000
 
         base &= 0xfffff000
         return base
@@ -818,6 +632,11 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
     def getSection(self, secname):
         return self.secnames.get(secname, None)
 
+    def getSectionByIndex(self, idx):
+        if idx >= len(self.sections):
+            return None
+        return self.sections[idx]
+
     def getSections(self):
         """
         Return the array of sections for this Elf
@@ -928,6 +747,14 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         '''
         return self.e_type == ET_EXEC
 
+    def isRelocatable(self):
+        '''
+        Returns true if the given Elf binary is marked as a relocatable file.
+        isRelocatable() helps determine if this ELF is a Kernel Module (.ko) 
+        or Object file (.o), *not* a Shared Object (.so) or executable.
+        '''
+        return self.e_type == ET_REL
+
     def __repr__(self, verbose=False):
         """
         Returns a string summary of this ELF.
@@ -968,7 +795,7 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
             mystr+= "\n"+repr(sym)
 
         mystr+= "\n\n= Relocation table:"
-        for reloc in self.relocs:
+        for _, reloc in self.relocs:
             mystr+= "\n"+repr(reloc)
 
         mystr+= "\n\n= Dynamics table:"
@@ -1017,6 +844,11 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         '''
         return self.symbols
 
+    def getSymbol(self, symidx):
+        if symidx is None or symidx >= len(self.symbols):
+            return None
+        return self.symbols[symidx]
+
     def getDynSymbol(self, symidx):
         '''
         Returns the DT_SYMTAB entry at index "symidx".
@@ -1068,12 +900,44 @@ class Elf(vs_elf.Elf32, vs_elf.Elf64):
         in self.dynsymtabct.  Perhaps this is horrible and should be stricken
         from the code.
         '''
-        #FIXME: make this use both SECTION and PT_DYNAMICS versions...
+        # if DT_SYMTAB doesn't exist, we don't have a symbol table defined by DYNAMICS
         symtabva = self.dyns.get(DT_SYMTAB)
         if symtabva is None:
             return None, None, None
 
         symsz = self.dyns.get(DT_SYMENT)
+
+        # calculate/reuse symbol count
+        if not self.dynsymtabct:    # check if we've already got this value
+            # calculate the Dynamic Symbol Table Size
+            self.dynsymtabct = len(self.dynstrtab)
+            if not self.dynsymtabct:
+                # if we haven't set up dynsymtabct, call _parseDynStrs and create one)
+                self._parseDynStrs()
+                self.dynsymtabct = len(self.dynstrtab)
+
+            # if "DT_SONAME" is within this string table, there are no symbols to match that or thereafter:
+            soname = self.dyns.get(DT_SONAME)
+            if soname is not None and soname != -1:
+                strsz = self.dyns.get(DT_STRSZ)
+                if soname < strsz:
+                    dynstrtabva = self.dyns.get(DT_STRTAB)
+                    strtabbytes = self.readAtRva(dynstrtabva, soname)
+                    dynsymstrs = strtabbytes.split(b'\0')
+                    self.dynsymtabct = len(dynsymstrs) - 1
+
+            # if any of the other Dyns (which are addresses) come after this, don't parse past them
+            for dyntype in dt_rebase:
+                dyn = self.dyns.get(dyntype)
+                if dyn is None:
+                    continue
+
+                if dyn > symtabva:
+                    if dyn < symtabva + (self.dynsymtabct * symsz):
+                        # we have a new winner!
+                        delta = dyn - symtabva
+                        self.dynsymtabct = delta // symsz
+
         count = self.dynsymtabct
         symtabsz = count * symsz
 
@@ -1109,10 +973,9 @@ def elfFromMemoryObject(memobj, baseaddr):
     fd = vstruct.MemObjFile(memobj, baseaddr)
     return Elf(fd)
 
-
+# TODO: Consider deprecating these. We have methods on the ElfReloc objects specifically for these two
 def getRelocType(val):
     return val & 0xff
 
-
 def getRelocSymTabIndex(val):
     return val >> 8
