@@ -272,18 +272,19 @@ class IntelEmulator(i386RegisterContext, envi.Emulator):
         ZF starts off being set. 
         Then the instruction is repeated and ECX decremented until either
         ECX reaches 0 or the ZF is cleared.
+        
+        Respects emu option "i386:repmax" limiting the number of repetitions.
         '''
         ecx = emu.getRegister(REG_ECX)
         emu.setFlag(EFLAGS_ZF, 1)
 
-        repmax = emu.getEmuOpt('i386:repmax')
-        if repmax:
-            ecx = min(ecx, repmax)
+        repmax = emu.getEmuOpt('i386:repmax') or ecx
 
         ret = None
-        while ecx and emu.getFlag(EFLAGS_ZF):
+        while ecx and repmax and emu.getFlag(EFLAGS_ZF):
             ret = meth(op)
             ecx -= 1
+            repmax -= 1
             emu.setRegister(REG_ECX, ecx)
         return ret
 
@@ -294,18 +295,19 @@ class IntelEmulator(i386RegisterContext, envi.Emulator):
         ZF starts off being cleared. 
         Then the instruction is repeated and ECX decremented until either
         ECX reaches 0 or the ZF is set.
+        
+        Respects emu option "i386:repmax" limiting the number of repetitions.
         '''
         ecx = emu.getRegister(REG_ECX)
         emu.setFlag(EFLAGS_ZF, 0)
 
-        repmax = emu.getEmuOpt('i386:repmax')
-        if repmax:
-            ecx = min(ecx, repmax)
+        repmax = emu.getEmuOpt('i386:repmax') or ecx
 
         ret = None
-        while ecx and not emu.getFlag(EFLAGS_ZF):
+        while ecx and repmax and not emu.getFlag(EFLAGS_ZF):
             ret = meth(op)
             ecx -= 1
+            repmax -= 1
             emu.setRegister(REG_ECX, ecx)
         return ret
 
@@ -2639,3 +2641,9 @@ class IntelEmulator(i386RegisterContext, envi.Emulator):
 
     def i_pextrd(self, op):
         self.i_pextrb(op, width=4)
+        
+    def i_vpcext(self, op):
+        # expected behavior: EBX is set to 0 if Virtual PC is detected or an exception is raised
+        # in a malware sample with an anti-vm check using this instruction, vpcext is followed by a "test ebx, ebx" and exception handling code. 
+        # to avoid a "non-defined" value in the register it is set to zero
+        self.setRegister(REG_EBX, 0xffffffff)
