@@ -2,7 +2,7 @@ import logging
 import binascii
 
 LOG_FORMAT = '%(asctime)s:%(levelname)s:%(name)s: %(message)s' \
-             '[%(filename)s:%(funcName)s:%(lineno)s:%(threadName)s]'
+             '[%(filename)s:%(funcName)s:%(lineno)s]'
 
 EMULOG = 9
 SHITE = 8
