@@ -4,7 +4,6 @@ Unified config object for all vtoys.
 
 import os
 import json
-import getpass
 import logging
 
 import envi.exc as e_exc
@@ -29,10 +28,6 @@ def gethomedir(*paths, **kwargs):
     return path
 
 
-def getusername():
-    return getpass.getuser()
-
-
 class EnviConfig:
     '''
     EnviConfig basically works like a multi-layer dictionary that
