@@ -66,5 +66,14 @@ class MapLookup:
                 return marray[ va - mva ]
         return None
 
+    def delMapLookup(self, va):
+        for midx in range(len(self._maps_list)):
+            mva, mvamax, marray = self._maps_list[midx]
+            if va >= mva and va < mvamax:
+                return self._maps_list.pop(midx)
+
+        raise e_exc.MapNotFoundException(va=va)
+
+
     def __getslice__(self, start, end):
         raise NotImplementedError("__getslice__ on MapLookup needs implementing")
