@@ -492,7 +492,8 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
             trace.setMeta('PendingBreak', False)
             bp = trace.getCurrentBreakpoint()
             if bp:
-                self.vprint("Thread: %d Hit Break: %s" % (tid, repr(bp)))
+                if not self.silent:
+                    self.vprint("Thread: %d Hit Break: %s" % (tid, repr(bp)))
                 cmdstr = self.bpcmds.get(bp.id, None)
                 if cmdstr is not None:
                     self.onecmd(cmdstr)
@@ -574,7 +575,7 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
         if dohex:
             memstr = binascii.unhexlify(memstr)
         if douni:
-            memstr = ("\x00".join(memstr)) + "\x00"
+            memstr = (b"\x00".join(memstr)) + b"\x00"
 
         addr = self.parseExpression(exprstr)
         self.memobj.writeMemory(addr, memstr)
@@ -1711,29 +1712,74 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
 
     def do_bpedit(self, line):
         """
-        Manipulcate the python code that will be run for a given
-        breakpoint by ID.  (Also the way to view the code).
+        Modify a given breakpoint.
+        * Manipulate the python code that will be run for a given
+          breakpoint by ID.
+        * Also the way to view the code
+        * Modify characteristics of the breakpoint (eg. FastBreak)
 
-        Usage: bpedit <id> ["optionally new code"]
+        Usage: bpedit [options] <id> ["optionally new code"]
+
+        where options include:
+           -F  toggles a breakpoint's FastBreak mode
+           -S  toggles a breakpoint's Silent mode (doesn't print console msg, still runs BP code)
+           -V  prints more metadata about a given breakpoint (default is just the code)
 
         NOTE: Your code must be surrounded by "s and may not
         contain any "s
         """
         argv = e_cli.splitargs(line)
+        verbose = False
+        try:
+            opts, args = getopt(argv, 'FSV')
+        except:
+            return self.do_help('bpedit')
+
         if len(argv) == 0:
             return self.do_help("bpedit")
-        bpid = int(argv[0])
 
-        if len(argv) == 2:
-            self.trace.setBreakpointCode(bpid, argv[1])
+        bpid = int(args[0])
 
+        bp = self.trace.getBreakpoint(bpid)
+        if bp is None:
+            self.vprint("Breakpoint %d does not exist!" % bpid)
+            return
+
+        for opt, optarg in opts:
+            if opt == '-V':
+                # print all the metadata
+                verbose = True
+
+            elif opt == '-S':
+                bp.silent = not bp.silent
+
+            elif opt == '-F':
+                bp.fastbreak = not bp.fastbreak
+
+        if len(args) == 2:
+            self.trace.setBreakpointCode(bpid, args[1])
+
+        # print the breakpoint metadata:
         pystr = self.trace.getBreakpointCode(bpid)
-        self.vprint("[%d] Breakpoint code: %s" % (bpid,pystr))
+        if verbose:
+            self.vprint("%r" % bp)
+            self.vprint("    code: %s" % (pystr))
+            self.vprint("    FastBreak:     %r" % bp.fastbreak)
+            self.vprint("    enabled:       %r" % bp.enabled)
+            self.vprint("    silent:        %r" % bp.silent)
+            self.vprint("    active:        %r" % bp.active)
+            self.vprint("    resonce:       %r" % bp.resonce)
+            if bp.stealthbreak:
+                self.vprint("    stealth:       %r" % bp.stealthbreak)
+
+        else:
+            self.vprint("[%d] Breakpoint code: %s" % (bpid,pystr))
 
     def do_bp(self, line):
         """
         Show, add,  and enable/disable breakpoints
         USAGE: bp [-d <addr>] [-a <addr>] [-o <addr>] [[-c pycode] <address> [vdb cmds]]
+        -A - Show *all* breakpoints (including special bp's used for VDB functionality)
         -C - Clear All Breakpoints
         -c "py code" - Set the breakpoint code to the given python string
         -d <id> - Disable Breakpoint
@@ -1762,12 +1808,13 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
 
         argv = e_cli.splitargs(line)
         try:
-            opts,args = getopt(argv, "fF:e:d:o:r:L:Cc:S:W:")
+            opts,args = getopt(argv, "AfF:e:d:o:r:L:Cc:S:W:")
         except Exception as e:
             return self.do_help('bp')
 
         pycode = None
         wpargs = None
+        showall = False
         fastbreak = False
         libsearch = None
 
@@ -1775,6 +1822,9 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
             if opt == "-e":
                 self.trace.setBreakpointEnabled(eval(optarg), True)
 
+            elif opt == "-A":
+                showall = True
+
             elif opt == "-c":
                 pycode = optarg
                 test = compile(pycode, "test","exec")
@@ -1793,6 +1843,9 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
 
             elif opt == "-C":
                 for bp in self.trace.getBreakpoints():
+                    if bp.stealthbreak:
+                        continue
+
                     self.bpcmds.pop(bp.id, None)
                     self.trace.removeBreakpoint(bp.id)
                     self.vdbUIEvent('vdb:delbreak', bp.id)
@@ -1861,6 +1914,9 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
 
         self.vprint(" [ Breakpoints ]")
         for bp in self.trace.getBreakpoints():
+            if bp.stealthbreak and not showall:
+                # don't list stealthbreak bp's (unless forced)
+                continue
             self._print_bp(bp)
 
     def _print_bp(self, bp):
@@ -2299,6 +2355,6 @@ class Vdb(e_cli.EnviMutableCli, v_notif.Notifier, v_util.TraceManager):
 ##############################################################################
 # The following are touched during the release process by bump2version.
 # You should have no reason to modify these yourself
-version = (1, 0, 7)
+version = (1, 1, 0)
 verstring = '.'.join([str(x) for x in version])
 commit = ''
