@@ -176,7 +176,7 @@ def buildFunctionApi(vw, fva, emu, emumon):
 
 
 def analyzeFunction(vw, fva):
-    emu = vw.getEmulator()
+    emu = vw.getEmulator(va=fva)
     emumon = AnalysisMonitor(vw, fva)
     emu.setEmulationMonitor(emumon)
 
