@@ -1,3 +1,5 @@
+import time
+import threading
 import functools
 import itertools
 import traceback
@@ -5,8 +7,10 @@ import collections
 
 import vqt.hotkeys as vq_hotkey
 import vqt.saveable as vq_save
+import envi.qt.memory as e_mem_qt
 import envi.memcanvas as e_memcanvas
 import envi.qt.memory as e_qt_memory
+import vivisect.qt.views as viv_q_views
 import envi.qt.memcanvas as e_qt_memcanvas
 
 import visgraph.layouts.dynadag as vg_dynadag
@@ -21,10 +25,12 @@ from PyQt5 import Qt, QtCore, QtGui, QtWebEngine, QtWidgets
 from PyQt5.QtCore import pyqtSignal
 from PyQt5.QtWidgets import *
 
+from envi.threads import firethread
 from vqt.main import idlethread, eatevents, workthread, vqtevent
 
 from vqt.common import *
 from vivisect.const import *
+from envi.common import MIRE
 
 class VQVivFuncgraphCanvas(vq_memory.VivCanvasBase):
     paintUp = pyqtSignal()
@@ -105,8 +111,14 @@ class VQVivFuncgraphCanvas(vq_memory.VivCanvasBase):
         runner = functools.partial(self._renderMemoryFinish, cb)
         e_memcanvas.MemoryCanvas.renderMemory(self, va, size, cb=runner)
 
-    def renderMemory(self, va, size, cb):
+    def renderMemory(self, va, size, cb=None):
+        '''
+        Funcgraph specific renderMemory() function.
+        '''
         # For the funcgraph canvas, this will be called once per code block
+        if not cb:
+            cb = self.__nopcb
+
         selector = 'codeblock_%.8x' % va
         js = '''var node = document.querySelector("#%s");
         if (node == null) {
@@ -118,11 +130,15 @@ class VQVivFuncgraphCanvas(vq_memory.VivCanvasBase):
         [%d, %d]
         ''' % (selector, selector, va, size)
         runner = functools.partial(self._renderMemoryCallback, cb)
+        logger.log(MIRE, "renderMemory(%r, %r) %r", va, cb, runner)
         self.page().runJavaScript(js, runner)
 
+    def __nopcb(self):
+        pass
+
     def contextMenuEvent(self, event):
         if self._canv_curva is not None:
-            menu = vq_ctxmenu.buildContextMenu(self.vw, va=self._canv_curva, parent=self)
+            menu = vq_ctxmenu.buildContextMenu(self.vw, va=self._canv_curva, parent=self, nav=self.parent())
         else:
             menu = QMenu(parent=self)
 
@@ -153,6 +169,8 @@ class VQVivFuncgraphCanvas(vq_memory.VivCanvasBase):
         self.page().runJavaScript(f'window.scroll({x}, {y})')
         eatevents()
 
+def reset():
+    VQVivFuncgraphView.viewidx = itertools.count()
 
 class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidget, vq_save.SaveableWidget, viv_base.VivEventCore):
     _renderDoneSignal = pyqtSignal()
@@ -164,10 +182,23 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         self.fva = None
         self.graph = None
         self.nodes = []
+        self._xref_cache = set()
+        self._xref_cache_bg = set()
+        self._xref_cache_lock = threading.Lock()
+
         self.vwqgui = vwqgui
         self._last_viewpt = None
+        self._rendering = False
+        self._renderWaiting = False
+        self._renderlock = threading.Lock()
         self.history = collections.deque((), 100)
 
+        self._autorefresh = None # created in getRendToolsMenu()
+
+        self._leading = False
+        self._following = None
+        self._follow_menu = None  # init'd in handler below
+
         QWidget.__init__(self, parent=vwqgui)
         vq_hotkey.HotKeyMixin.__init__(self)
         viv_base.VivEventCore.__init__(self, vw)
@@ -189,6 +220,9 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
 
         self.addr_entry = QLineEdit(parent=self.top_box)
 
+        self.rend_tools = QPushButton('Opts', parent=self.top_box)
+        self.rend_tools.setMenu( self.getRendToolsMenu() )
+
         self.mem_canvas = VQVivFuncgraphCanvas(vw, syms=vw, parent=self)
         self.mem_canvas.setNavCallback(self.enviNavGoto)
         self.mem_canvas.refreshSignal.connect(self.refresh)
@@ -202,6 +236,7 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
 
         hbox.addWidget(self.hist_button)
         hbox.addWidget(self.addr_entry)
+        hbox.addWidget(self.rend_tools)
 
         vbox = QVBoxLayout(self)
         vbox.setContentsMargins(4, 4, 4, 4)
@@ -220,7 +255,7 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
 
         QtWidgets.QShortcut(QtGui.QKeySequence("Escape"), self, activated=self._hotkey_histback, context=3)
 
-        # TODO: Transition theses to the above pattern (since escape/ctrl-c
+        # TODO: Transition these to the above pattern (since escape/ctrl-c)
         # See: https://stackoverflow.com/questions/56890831/qwidget-cannot-catch-escape-backspace-or-c-x-key-press-events
         self.addHotKey('ctrl+0', 'funcgraph:resetzoom')
         self.addHotKeyTarget('funcgraph:resetzoom', self._hotkey_resetzoom)
@@ -236,6 +271,138 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         self.addHotKeyTarget('funcgraph:paintdown', self._hotkey_paintDown)
         self.addHotKey('ctrl+m', 'funcgraph:paintmerge')
         self.addHotKeyTarget('funcgraph:paintmerge', self._hotkey_paintMerge)
+        self.addHotKey('x', 'viv:xrefsto')
+        self.addHotKeyTarget('viv:xrefsto', self._viv_xrefsto)
+
+    def toggleAutoRefresh(self):
+        '''
+        This allows the function graph to auto-refresh on changes (or not)
+        '''
+        self._autorefresh.setChecked(not self._autorefresh.isChecked())
+
+    def _canHazAutoRefresh(self):
+        '''
+        Should we autorefresh on updates?
+        '''
+        return self._autorefresh.isChecked()
+
+    def rendToolsSetName(self, user=None):
+        curname = self.getEnviNavName()
+        mwname, ok = QInputDialog.getText(self, 'Set Mem Window Name', 'Name', text=curname)
+        if ok:
+            self.setMemWindowName(str(mwname))
+
+        if self.vw.server and self._leading:
+            if user is None:
+                user = self.vw.config.user.name
+            self.vw.modifyLeaderSession(self.uuid, user, mwname)
+        
+    def rendToolsMenu(self, event):
+        menu = self.getRendToolsMenu()
+        menu.exec_(self.mapToGlobal(self.rend_tools.pos()))
+
+    def getRendToolsMenu(self):
+        menu = QMenu(parent=self.rend_tools)
+        menu.addAction('set name', self.rendToolsSetName)
+
+        self._autorefresh = QAction('auto-refresh', menu, checkable=True, checked=True)
+        menu.addAction(self._autorefresh)
+
+        if self.vw.server:
+            leadact = QAction('lead', menu, checkable=True)
+
+            def leadToggle():
+                self._leading = not self._leading
+                # We can only follow if not leading... (deep huh? ;) )
+                self._follow_menu.setEnabled(not self._leading)
+                if self._leading:
+                    self._following = None
+                    locexpr = self.addr_entry.text()
+                    self.vw.iAmLeader(self.uuid, self.getEnviNavName(), locexpr)
+                else:
+                    self.vw.killLeaderSession(self.uuid)
+                self.updateWindowTitle()
+
+            def clearFollow():
+                self._following = None
+                self.updateWindowTitle()
+
+            leadact.toggled.connect(leadToggle)
+            menu.addAction(leadact)
+            self._follow_menu = menu.addMenu('Follow..')
+
+            self._rtmRebuild()
+
+        return menu
+
+    def _rtmAddLeaderSession(self, uuid, user, fname):
+        '''
+        Add Action to RendToolsMenu (Opts/Follow) for a given session
+        '''
+        logger.info("_rtmAddLeaderSession(%r, %r, %r)", uuid, user, fname)
+
+        def setFollow():
+            self._following = uuid
+            self.updateWindowTitle()
+            self.navToLeader()
+
+        action = self._follow_menu.addAction('%s - %s' % (user, fname), setFollow)
+        action.setWhatsThis(uuid)
+
+    def _rtmModLeaderSession(self, uuid, user, fname):
+        '''
+        Add Action to RendToolsMenu (Opts/Follow) for a given session
+        '''
+        logger.info("_rtmModLeaderSession(%r, %r, %r)", uuid, user, fname)
+
+        action = self._rtmGetActionByUUID(uuid)
+        action.setText('%s - %s' % (user, fname))
+        if self._following == uuid:
+            self.updateWindowTitle()
+
+    def _rtmGetActionByUUID(self, uuid):
+        for action in self._follow_menu.actions():
+            if action.whatsThis() == uuid:
+                return action
+
+        logger.info("Unknown UUID: %r" % uuid)
+
+    def _rtmDelLeaderSession(self, uuid):
+        '''
+        Remove Action item from RendToolsMenu (Opts/Follow)
+        '''
+        action = self._rtmGetActionByUUID(uuid)
+        if action:
+            self._follow_menu.removeAction(action)
+            logger.info("Removing %r from Follow menu (%r)" % (action, uuid))
+            if self._following == uuid:
+                self._following = None
+                self.updateWindowTitle()
+        else:
+            logger.warning("Attempting to remove Menu Action that doesn't exist: %r", uuid)
+            self._rtmRebuild()
+
+    def _rtmRebuild(self):
+        '''
+        Sync current menu system with vw.leaders
+        '''
+        self._follow_menu.clear()
+
+        def clearFollow():
+            self._following = None
+            self.updateWindowTitle()
+
+        self._follow_menu.addAction('(disable)', clearFollow)
+
+        # add in the already existing sessions...
+        for uuid, (user, fname) in self.vw.getLeaderSessions().items():
+            self._rtmAddLeaderSession(uuid, user, fname)
+
+    def _rtmGetUUIDs(self):
+        '''
+        Returns a list of active UUIDs for leader sessions
+        '''
+        return [action.whatsThis() for action in self._follow_menu.actions()]
 
     def _nav_expr(self):
         expr = self.addr_entry.text()
@@ -274,6 +441,20 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
 
         self.mem_canvas.setZoomFactor(newzoom)
 
+    def _viv_xrefsto(self):
+
+        if self.mem_canvas._canv_curva is not None:
+            xrefs = self.vw.getXrefsTo(self.mem_canvas._canv_curva)
+            if len(xrefs) == 0:
+                self.vw.vprint('No xrefs found!')
+                return
+
+            title = 'Xrefs To: 0x%.8x' % self.mem_canvas._canv_curva
+            view = viv_q_views.VQXrefView(self.vw, self.vwqgui, xrefs=xrefs, title=title)
+            dock = self.vwqgui.vqDockWidget(view, floating=True)
+            dock.resize(800, 600)
+
+    @firethread
     def refresh(self):
         '''
         Cause the Function Graph to redraw itself.
@@ -282,8 +463,26 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         that have changed since last update, and be fast, so we can update
         after every change.
         '''
+        # debounce logic
+        if self._renderWaiting:
+            # if we are already rendering... and another thing is waiting
+            #logger.debug('(%r) someone is already waiting... returning', threading.currentThread())
+            return
+
+        while self._rendering:
+            #logger.debug('(%r) waiting for render to complete...', threading.currentThread())
+            self._renderWaiting = True
+            time.sleep(.1)
+
+        self._renderWaiting = False
+
+        with self._renderlock:
+            #logger.debug('(%r) render beginning...', threading.currentThread())
+            self._rendering = True
+
+        # actually do the refresh
         self._last_viewpt = self.mem_canvas.page().scrollPosition()
-        # FIXME: history should track this as well and return to the same place
+        # TODO: history should track this as well and return to the same place
         self.clearText()
         self.fva = None
         self._renderMemory()
@@ -292,7 +491,7 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
     def _refresh_cb(self):
         '''
         This is a hack to make sure that when _renderMemory() completes,
-        _refresh_3() gets run after all other rendering events yet to come.
+        _refresh_cb() gets run after all other rendering events yet to come.
         '''
         if self._last_viewpt is None:
             return
@@ -300,6 +499,14 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         self.mem_canvas.setScrollPosition(self._last_viewpt.x(), self._last_viewpt.y())
         self._last_viewpt = None
 
+        with self._renderlock:
+            self._rendering = False
+        #logger.debug('(%r) render finished!', threading.currentThread())
+
+        # update the "real" _xref_cache from the one we just built
+        with self._xref_cache_lock:
+            self._xref_cache = self._xref_cache_bg.copy()
+
     def _histSetupMenu(self):
         self.histmenu.clear()
 
@@ -322,12 +529,120 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
             pass
         self.enviNavGoto(expr)
 
+    def isReferenced(self, va):
+        '''
+        During function building, we can grab any xrefs from this function and cache them
+        This is the easy way to determine if an address is referenced from this function
+        '''
+        with self._xref_cache_lock:
+            if va in self._xref_cache:
+                return True
+
+        # catch any "stragglers"
+        if va in self._xref_cache_bg:
+            return True
+
+        return False
+
+    def _refreshIfNecessary(self, va):
+        '''
+        Called from VWE event handlers with updated VA
+        '''
+        if not self._canHazAutoRefresh():
+            return 
+
+        if self.vw.getFunction(va) == self.fva:
+            self.refresh()
+
+        if self.isReferenced(va):
+            self.refresh()
+
+    def VWE_SYMHINT(self, vw, event, einfo):
+        va, idx, hint = einfo
+        self.mem_canvas.renderMemoryUpdate(va, 1)
+        self._refreshIfNecessary(va)
+
+    def VWE_ADDLOCATION(self, vw, event, einfo):
+        va, size, ltype, tinfo = einfo
+        self.mem_canvas.renderMemoryUpdate(va, size)
+        self._refreshIfNecessary(va)
+
+    def VWE_DELLOCATION(self, vw, event, einfo):
+        va, size, ltype, tinfo = einfo
+        self.mem_canvas.renderMemoryUpdate(va, size)
+        self._refreshIfNecessary(va)
+
+    def VWE_ADDFUNCTION(self, vw, event, einfo):
+        va, meta = einfo
+        self.mem_canvas.renderMemoryUpdate(va, 1)
+        self._refreshIfNecessary(va)
+
+    def VWE_SETFUNCMETA(self, vw, event, einfo):
+        fva, key, val = einfo
+        self._refreshIfNecessary(fva)
+
+    def VWE_SETFUNCARGS(self, vw, event, einfo):
+        fva, fargs = einfo
+        self._refreshIfNecessary(fva)
+
+    def VWE_COMMENT(self, vw, event, einfo):
+        va, cmnt = einfo
+        self.mem_canvas.renderMemoryUpdate(va, 1)
+        self._refreshIfNecessary(va)
+
+    @idlethread
+    def VWE_SETNAME(self, vw, event, einfo):
+        va, name = einfo
+        self.mem_canvas.renderMemoryUpdate(va, 1)
+        for fromva, tova, rtype, rflag in self.vw.getXrefsTo(va):
+            self.mem_canvas.renderMemoryUpdate(fromva, 1)
+        self._refreshIfNecessary(va)
+
+    @idlethread
+    def VTE_IAMLEADER(self, vw, event, einfo):
+        # we have to maintain the list here, since the menu itself is not 
+        # autogenerated each time, like a context menu.
+        uuid, user, fname, locexpr = einfo
+        self._rtmAddLeaderSession(uuid, user, fname)
+
+    @idlethread
+    def VTE_KILLLEADER(self, vw, event, einfo):
+        uuid = einfo
+        self._rtmDelLeaderSession(uuid)
+
+    @idlethread
+    def VTE_FOLLOWME(self, vw, event, einfo):
+        uuid, expr = einfo
+        if self._following != uuid:
+            return
+        self.enviNavGoto(expr)
+
+    @idlethread
+    def VTE_MODLEADER(self, vw, event, einfo):
+        uuid, user, fname = einfo
+        self._rtmModLeaderSession(uuid, user, fname)
+
+    @idlethread
     def enviNavGoto(self, expr, sizeexpr=None):
+        if self._leading:
+            self.vw.followTheLeader(self.uuid, str(expr))
+
         self.addr_entry.setText(expr)
         self.history.append( expr )
         self.updateWindowTitle()
         self._renderMemory()
 
+    @idlethread
+    def navToLeader(self):
+        uuid = self._following
+        logger.debug("navToLeader(%r)", uuid)
+        if uuid:
+            expr = self.vw.getLeaderLoc(uuid)
+            if not expr:
+                return
+
+            self.enviNavGoto(expr)
+
     def vqGetSaveState(self):
         return { 'expr':str(self.addr_entry.text()), }
 
@@ -342,16 +657,38 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         self.setEnviNavName(mwname)
         self.updateWindowTitle()
 
-    def updateWindowTitle(self, data=None):
-        ename = self.getEnviNavName()
+    def getExprTitle(self):
+        va = -1
         expr = str(self.addr_entry.text())
+
         try:
+
             va = self.vw.parseExpression(expr)
             smartname = self.vw.getName(va, smart=True)
-            self.setWindowTitle('%s: %s (0x%x)' % (ename, smartname, va))
-            return va
-        except:
-            self.setWindowTitle('%s: %s (0x----)' % (ename, expr))
+            title = '%s (0x%x)' % (smartname, va)
+
+            name = self.vw.getName(va)
+            if name is not None:
+                title = name
+
+        except Exception:
+            title = 'expr error'
+
+        if self._leading:
+            title += ' (leading)'
+
+        if self._following is not None:
+            uuid = self._following
+            user, window = self.vw.getLeaderInfo(uuid)
+            title += ' (following %s %s)' % (user, window)
+
+        return title, va
+
+    def updateWindowTitle(self, data=None):
+        ename = self.getEnviNavName()
+        expr, va = self.getExprTitle()
+        self.setWindowTitle('%s: %s' % (ename, expr))
+        return va
 
     # DEV: None of these methods are meant to be called directly by anybody but themselves,
     # since they're setup in a way to make renderFunctionGraph play nicely with pyqt5
@@ -363,6 +700,7 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         addr = self.updateWindowTitle()
         if addr is not None:
             vqtevent('viv:colormap', {addr: 'orange'})
+
         self._renderDoneSignal.emit()
 
     def _edgesDone(self, data):
@@ -531,10 +869,23 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
         some safety rails if the user switches functions in the middle of rendering
         '''
         if len(self.nodes):
+            # render codeblock
             node = self.nodes.pop(0)
             cbva = node[1].get('cbva')
             cbsize = node[1].get('cbsize')
             self.mem_canvas.renderMemory(cbva, cbsize, self._renderCodeBlock)
+
+            # update _xref_cache_bg (made "real" when rendering complete)
+            endva = cbva + cbsize
+            while cbva < endva:
+                for xref in self.vw.getXrefsFrom(cbva):
+                    xrfr, xrto, xrtype, xrflag = xref
+                    self._xref_cache_bg.add(xrto)
+                    #logger.debug("adding 0x%x -> 0x%x to _xref_cache", xrfr, xrto)
+
+                lva, lsz, ltp, ltinfo = self.vw.getLocation(cbva)
+                cbva += lsz
+
         else:
             self._getNodeSizes()
 
@@ -569,13 +920,12 @@ class VQVivFuncgraphView(vq_hotkey.HotKeyMixin, e_qt_memory.EnviNavMixin, QWidge
 
         self.graph = graph
 
-        # Go through each of the nodes and render them so we know sizes
+        # Clear _xref_cache
+        self._xref_cache_bg.clear()
+
+        # Go through each of the nodes and render them so we know sizes (and xrefs)
         self.nodes = self.graph.getNodes()
-        if len(self.nodes):
-            node = self.nodes.pop(0)
-            cbva = node[1].get('cbva')
-            cbsize = node[1].get('cbsize')
-            self.mem_canvas.renderMemory(cbva, cbsize, self._renderCodeBlock)
+        self._renderCodeBlock(None)
 
     def _renderedSameFva(self, data):
         addr = self.updateWindowTitle()
