@@ -23,7 +23,6 @@ logger = logging.getLogger(__name__)
 qt_horizontal   = 1
 qt_vertical     = 2
 
-
 class VivCanvasBase(vq_hotkey.HotKeyMixin, e_mem_canvas.VQMemoryCanvas):
 
     def __init__(self, *args, **kwargs):
@@ -42,9 +41,10 @@ class VivCanvasBase(vq_hotkey.HotKeyMixin, e_mem_canvas.VQMemoryCanvas):
         self.addHotKey('n', 'viv:setname')
         self.addHotKey('g', 'viv:getlocation')
         self.addHotKey(';', 'viv:comment')
-        self.addHotKey('S', 'viv:make:struct')
-        self.addHotKey('ctrl+S', 'viv:make:struct:again')
-        self.addHotKey('ctrl+meta+S', 'viv:make:struct:multi')
+        self.addHotKey(':', 'viv:commentmulti')
+        self.addHotKey('t', 'viv:make:struct')
+        self.addHotKey('ctrl+t', 'viv:make:struct:again')
+        self.addHotKey('ctrl+meta+t', 'viv:make:struct:multi')
         self.addHotKey('U', 'viv:undefine')
         self.addHotKey('ctrl+p', 'viv:preview:instr')
         self.addHotKey('B', 'viv:bookmark')
@@ -173,6 +173,11 @@ class VivCanvasBase(vq_hotkey.HotKeyMixin, e_mem_canvas.VQMemoryCanvas):
     @vq_hotkey.hotkey('viv:make:code')
     def _hotkey_make_code(self):
         if self._canv_curva is not None:
+            loctup = self.vw.getLocation(self._canv_curva)
+            if loctup is not None:
+                self.vw.vprint("Failed to make code (at 0x%x) where something already exists: %r " % (va, self.reprLocation(loctup)))
+                return
+
             self.vw.makeCode(self._canv_curva)
 
     @vq_hotkey.hotkey('viv:make:function')
@@ -184,16 +189,31 @@ class VivCanvasBase(vq_hotkey.HotKeyMixin, e_mem_canvas.VQMemoryCanvas):
     @vq_hotkey.hotkey('viv:make:string')
     def _hotkey_make_string(self):
         if self._canv_curva is not None:
+            loctup = self.vw.getLocation(self._canv_curva)
+            if loctup is not None:
+                self.vw.vprint("Failed to make string (at 0x%x) where something already exists: %r " % (va, self.reprLocation(loctup)))
+                return
+
             self.vw.makeString(self._canv_curva)
 
     @vq_hotkey.hotkey('viv:make:pointer')
     def _hotkey_make_pointer(self):
         if self._canv_curva is not None:
+            loctup = self.vw.getLocation(self._canv_curva)
+            if loctup is not None:
+                self.vw.vprint("Failed to make pointer (at 0x%x) where something already exists: %r " % (va, self.reprLocation(loctup)))
+                return
+
             self.vw.makePointer(self._canv_curva)
 
     @vq_hotkey.hotkey('viv:make:unicode')
     def _hotkey_make_unicode(self):
         if self._canv_curva is not None:
+            loctup = self.vw.getLocation(self._canv_curva)
+            if loctup is not None:
+                self.vw.vprint("Failed to make unicode (at 0x%x) where something already exists: %r " % (va, self.reprLocation(loctup)))
+                return
+
             self.vw.makeUnicode(self._canv_curva)
 
     @vq_hotkey.hotkey('viv:undefine')
@@ -221,6 +241,11 @@ class VivCanvasBase(vq_hotkey.HotKeyMixin, e_mem_canvas.VQMemoryCanvas):
         if self._canv_curva is not None:
             self.vw.getVivGui().setVaComment(self._canv_curva, parent=self)
 
+    @vq_hotkey.hotkey('viv:commentmulti')
+    def _hotkey_commentmulti(self):
+        if self._canv_curva is not None:
+            self.vw.getVivGui().setVaMultilineComment(self._canv_curva, parent=self)
+
     @vq_hotkey.hotkey('viv:make:struct')
     def _hotkey_make_struct(self):
         if self._canv_curva is not None:
@@ -402,7 +427,10 @@ class VQVivMemoryView(e_mem_qt.VQMemoryWindow, viv_base.VivEventCore):
                 self._follow_menu.setEnabled(not self._leading)
                 if self._leading:
                     self._following = None
-                    self.vw.iAmLeader(self.mwname)
+                    locexpr = self.addr_entry.text()
+                    self.vw.iAmLeader(self.uuid, self.getEnviNavName(), locexpr)
+                else:
+                    self.vw.killLeaderSession(self.uuid)
                 self.updateMemWindowTitle()
 
             def clearFollow():
@@ -414,9 +442,28 @@ class VQVivMemoryView(e_mem_qt.VQMemoryWindow, viv_base.VivEventCore):
             self._follow_menu = menu.addMenu('Follow..')
             self._follow_menu.addAction('(disable)', clearFollow)
 
+            # add in the already existing sessions...
+            for uuid, (user, fname) in self.vw.getLeaderSessions().items():
+
+                def setFollow():
+                    self._following = uuid
+                    self.updateMemWindowTitle()
+                    self.navToLeader()
+
+                action = self._follow_menu.addAction('%s - %s' % (user, fname), setFollow)
+                action.setWhatsThis(uuid)
+
         return menu
 
+    def rendToolsSetName(self, user=None):
+        menu = e_mem_qt.VQMemoryWindow.rendToolsSetName(self)
+        if self.vw.server and self._leading:
+            if user is None:
+                user = self.vw.config.user.name
+            self.vw.modifyLeaderSession(self.uuid, user, self.mwname)
+        
     def getExprTitle(self):
+        va = -1
         title = str(self.addr_entry.text())
 
         try:
@@ -433,10 +480,11 @@ class VQVivMemoryView(e_mem_qt.VQMemoryWindow, viv_base.VivEventCore):
             title += ' (leading)'
 
         if self._following is not None:
-            user, window = self._following
+            uuid = self._following
+            user, window = self.vw.getLeaderInfo(uuid)
             title += ' (following %s %s)' % (user, window)
 
-        return title
+        return title, va
 
     def _getRenderVaSize(self):
         '''
@@ -490,8 +538,74 @@ class VQVivMemoryView(e_mem_qt.VQMemoryWindow, viv_base.VivEventCore):
         for cbva, cbsize, cbfva in self.vw.getFunctionBlocks(fva):
             self.mem_canvas.renderMemoryUpdate(cbva, cbsize)
 
-    def VTE_IAMLEADER(self, vw, event, einfo):
-        user, followname = einfo
+    def _rtmAddLeaderSession(self, uuid, user, fname):
+        '''
+        Add Action to RendToolsMenu (Opts/Follow) for a given session
+        '''
+        logger.info("_rtmAddLeaderSession(%r, %r, %r)", uuid, user, fname)
+
+        def setFollow():
+            self._following = uuid
+            self.updateMemWindowTitle()
+            self.navToLeader()
+
+        action = self._follow_menu.addAction('%s - %s' % (user, fname), setFollow)
+        action.setWhatsThis(uuid)
+
+    def _rtmModLeaderSession(self, uuid, user, fname):
+        '''
+        Add Action to RendToolsMenu (Opts/Follow) for a given session
+        '''
+        logger.info("_rtmModLeaderSession(%r, %r, %r)", uuid, user, fname)
+        
+        action = self._rtmGetActionByUUID(uuid)
+        action.setText('%s - %s' % (user, fname))
+        if self._following == uuid:
+            self.updateMemWindowTitle()
+
+    def _rtmGetActionByUUID(self, uuid):
+        for action in self._follow_menu.actions():
+            if action.whatsThis() == uuid:
+                return action
+
+        logger.info("Unknown UUID: %r" % uuid)
+
+    def _rtmDelLeaderSession(self, uuid):
+        '''
+        Remove Action item from RendToolsMenu (Opts/Follow)
+        '''
+        action = self._rtmGetActionByUUID(uuid)
+        if action:
+            logger.info("Removing %r from Follow menu (%r)" % (action, uuid))
+            self._follow_menu.removeAction(action)
+            if self._following == uuid:
+                self._following = None
+                self.updateMemWindowTitle()
+        else:
+            logger.warning("Attempting to remove Menu Action that doesn't exist: %r", uuid)
+            self._rtmRebuild()
+
+    def _rtmRebuild(self):
+        '''
+        Sync current menu system with vw.leaders
+        '''
+        self._follow_menu.clear()
+
+        def clearFollow():
+            self._following = None
+            self.updateWindowTitle()
+
+        self._follow_menu.addAction('(disable)', clearFollow)
+
+        # add in the already existing sessions...
+        for uuid, (user, fname) in self.vw.getLeaderSessions().items():
+            self._rtmAddLeaderSession(uuid, user, fname)
+
+    def _rtmGetUUIDs(self):
+        '''
+        Returns a list of active UUIDs for leader sessions
+        '''
+        return [action.whatsThis() for action in self._follow_menu.actions()]
 
     def VWE_SYMHINT(self, vw, event, einfo):
         va, idx, hint = einfo
@@ -530,23 +644,39 @@ class VQVivMemoryView(e_mem_qt.VQMemoryWindow, viv_base.VivEventCore):
 
     @idlethread
     def VTE_IAMLEADER(self, vw, event, einfo):
-        user, fname = einfo
+        uuid, user, fname, locexpr = einfo
+        self._rtmAddLeaderSession(uuid, user, fname)
 
-        def setFollow():
-            self._following = einfo
-            self.updateMemWindowTitle()
+    @idlethread
+    def VTE_KILLLEADER(self, vw, event, einfo):
+        uuid = einfo
+        self._rtmDelLeaderSession(uuid)
 
-        self._follow_menu.addAction('%s - %s' % (user, fname), setFollow)
+    @idlethread
+    def VTE_MODLEADER(self, vw, event, einfo):
+        uuid, user, fname = einfo
+        self._rtmModLeaderSession(uuid, user, fname)
 
     @idlethread
     def VTE_FOLLOWME(self, vw, event, einfo):
-        user, fname, expr = einfo
-        if self._following != (user, fname):
+        uuid, expr = einfo
+        if self._following != uuid:
             return
         self.enviNavGoto(expr)
 
     @idlethread
     def enviNavGoto(self, expr, sizeexpr='256', rend=''):
         if self._leading:
-            self.vw.followTheLeader(str(self.mwname), str(expr))
+            self.vw.followTheLeader(str(self.uuid), str(expr))
         return e_mem_qt.VQMemoryWindow.enviNavGoto(self, expr, sizeexpr=sizeexpr, rend=rend)
+
+    @idlethread
+    def navToLeader(self):
+        uuid = self._following
+        logger.debug("navToLeader(%r)" % uuid)
+        if uuid:
+            expr = self.vw.getLeaderLoc(uuid)
+            if not expr:
+                return
+
+            self.enviNavGoto(expr)
