@@ -16,6 +16,9 @@ class MockVw(object):
     def getLocation(self, va):
         return self._locs.get(va, None)
 
+    def getPointerSize(self):
+        return self.psize
+
 
 def getTestBytes(*paths):
     testdir = os.getenv('VIVTESTFILES')
@@ -42,13 +45,36 @@ def getTestPath(*paths):
 
 
 @functools.lru_cache()
-def getTestWorkspace(*paths):
+def getTestWorkspace(*paths, vw=None):
     testdir = os.getenv('VIVTESTFILES')
     if not testdir:
         raise unittest.SkipTest('VIVTESTFILES env var not found!')
+
     testdir = os.path.abspath(testdir)
     fpath = os.path.join(testdir, *paths)
-    vw = v_cli.VivCli()
+
+    if not vw:
+        vw = v_cli.VivCli()
+        vw.config.viv.analysis.symswitchcase.timeout_secs = 30
+
+
+    vw.loadFromFile(fpath)
+    vw.analyze()
+    return vw
+
+def getTestWorkspace_nocache(*paths, vw=None):
+    testdir = os.getenv('VIVTESTFILES')
+    if not testdir:
+        raise unittest.SkipTest('VIVTESTFILES env var not found!')
+
+    testdir = os.path.abspath(testdir)
+    fpath = os.path.join(testdir, *paths)
+
+    if not vw:
+        vw = v_cli.VivCli()
+        vw.config.viv.analysis.symswitchcase.timeout_secs = 30
+
+
     vw.loadFromFile(fpath)
     vw.analyze()
     return vw
