@@ -7,8 +7,8 @@ import argparse
 import cProfile
 import importlib.util
 
+import envi.exc as e_exc
 import envi.common as e_common
-import envi.config as e_config
 import envi.threads as e_threads
 
 import vivisect.cli as viv_cli
@@ -44,6 +44,8 @@ def main():
                         help='Path to a directory to use for config data')
     parser.add_argument('-a', '--autosave', dest='autosave', default=False, action='store_true',
                         help='Autosave configuration data')
+    parser.add_argument('-o', '--outfile', default=None,
+                        help='Name of VivWorkspace file to create (useful for loading multiple binaries into one workspace)')
     parser.add_argument('file', nargs='*')
     args = parser.parse_args()
 
@@ -54,6 +56,9 @@ def main():
     level = e_common.LOG_LEVELS[vw.verbose]
     e_common.initLogging(logger, level=level)
 
+    if args.outfile:
+        vw.setMeta('StorageName', args.outfile)
+
 
     # do things
     if args.option is not None:
@@ -64,7 +69,7 @@ def main():
 
             try:
                 vw.config.parseConfigOption(option)
-            except e_config.ConfigNoAssignment as e:
+            except e_exc.ConfigNoAssignment as e:
                 logger.critical(vw.config.reprConfigPaths() + "\n")
                 logger.critical(e)
                 logger.critical("syntax: \t-O <secname>.<optname>=<optval> (optval must be json syntax)")
